@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-# $Id: Getopt.pm,v 1.61 2008/01/06 18:35:49 eserte Exp $
+# $Id: Getopt.pm,v 1.64 2008/02/08 22:23:56 eserte Exp $
 # Author: Slaven Rezic
 #
 # Copyright (C) 1997,1998,1999,2000,2003,2007,2008 Slaven Rezic. All rights reserved.
@@ -24,7 +24,9 @@ use constant OPTTYPE  => 1;
 use constant DEFVAL   => 2;
 use constant OPTEXTRA => 3;
 
-$VERSION = '0.49_53';
+use Carp qw();
+
+$VERSION = '0.49_54';
 $VERSION = eval $VERSION;
 
 $DEBUG = 0;
@@ -131,9 +133,10 @@ sub new {
 	die "No opttable array ref or getopt hash ref";
     }
 
-    $self->{'caller'}   = (caller)[0];
-    $self->{'filename'} = delete $a{'-filename'};
-    $self->{'nosafe'}   = delete $a{'-nosafe'};
+    $self->{'caller'}         = (caller)[0];
+    $self->{'filename'}       = delete $a{'-filename'};
+    $self->{'nosafe'}         = delete $a{'-nosafe'};
+    $self->{'useerrordialog'} = delete $a{'-useerrordialog'};
 
     die "Unrecognized arguments: " . join(" ", %a) if %a;
 
@@ -247,7 +250,7 @@ sub save_options {
     eval "require Data::Dumper";
     if ($@) {
 	warn $@;
-	die "No Data::Dumper\n";
+	$self->my_die("No Data::Dumper available, cannot save options.\n");
     } else {
 	if (open(OPT, ">$filename")) {
 	    my %saveoptions;
@@ -281,8 +284,7 @@ sub save_options {
 	    warn "Options written to $filename" if $DEBUG;
 	    1;
 	} else {
-	    warn "Can't write to $filename";
-	    die "Writing failed\n";
+	    $self->my_die("Writing to config file <$filename> failed: $!\n");
 	    undef;
 	}
     }
@@ -433,6 +435,42 @@ sub process_options {
     }
 }
 
+sub my_die {
+    my($self, $msg, $is_safe) = @_;
+    my $use_tk;
+    if ($self->{'useerrordialog'} && defined &Tk::MainWindow::Existing) {
+	for my $mw (Tk::MainWindow::Existing()) {
+	    if (Tk::Exists($mw)) {
+		$use_tk = $mw;
+		last;
+	    }
+	}
+	if ($use_tk && !defined $is_safe) {
+	    for(my $i=0; $i<100; $i++) {
+		my(undef,undef,undef,$subroutine) = caller($i);
+		last if !defined $subroutine;
+		if ($subroutine eq '(eval)') {
+		    $use_tk = 0;
+		    last;
+		}
+	    }
+	}
+    }
+    if ($use_tk) {
+	eval {
+	    $use_tk->messageBox(-icon    => "error",
+				-message => $msg,
+				-title   => "Error",
+			       );
+	};
+	if ($@) {
+	    Carp::croak($msg);
+	}
+    } else {
+	Carp::croak($msg);
+    }
+}
+
 # try to work around weird browse entry
 sub _fix_layout {
     my($self, $frame, $widget, %args) = @_;
@@ -1203,7 +1241,7 @@ sub option_editor {
 			     my $err = $@;
 			     $top->Unbusy;
 			     if ($err) {
-				 die $err;
+				 $self->my_die($err, 'safe');
 			     }
                              $opt_editor->destroy;
 			     $$button_pressed = 'ok';
@@ -1510,6 +1548,15 @@ options.
 If set to true, do not use a safe compartment when loading options
 (see B<load_options>).
 
+=item -useerrordialog
+
+If set to true, then use an error dialog in user-relevant error
+conditions. Otherwise, the error message is printed to STDERR. This
+only includes errors which may happen in normal operation, but not
+programming errors like specifying erroneous options. If no Tk context
+is available (i.e. there is no MainWindow), then the error message
+will also be printed to STDERR.
+
 =back
 
 =item B<set_defaults>
